/*
 * @(#)JavaObjectRef.java	1.5 98/10/08
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hat.model;

/**
 * A forward reference to an object.  This is an intermediate representation
 * for a JavaThing, when we have the thing's ID, but we might not have read
 * the thing yet.
 *
 * @version     1.5, 10/08/98
 * @author      Bill Foote
 */




public class JavaObjectRef extends JavaThing {

    private int id;

    public JavaObjectRef(int id) {
	this.id = id;
    }


    public boolean isHeapAllocated() {
	return true;
    }

    public JavaThing dereference(Snapshot snapshot, JavaField field) {
	if (field != null && !field.hasId()) {
	    // If this happens, we must be a field that represents an int.
	    // (This only happens with .bod-style files)
	    return new JavaInt(id);
	}
	if (id == 0) {
	    return snapshot.getNullThing();
	}
	JavaThing result = snapshot.findThing(id);
	if (result == null) {
	    if (!snapshot.getUnresolvedObjectsOK()) {
		String msg = "Warning!  Failed to resolve object id 0x"
				+ hat.util.Misc.toHex(id);
		if (field != null) {
		    msg += " for field " + field.getName()
			    + " (signature " + field.getSignature() + ")";
		}
		System.out.println(msg);
		Thread.dumpStack();
	    }
	    result = new HackJavaValue("Unresolved object 0x" 
					+ hat.util.Misc.toHex(id), 0);
	}
	return result;
    }

    public int getSize() {
	return 0;
    }

    public String toString() {
	return "Unresolved object 0x" + hat.util.Misc.toHex(id);
    }
}
