/*
 * @(#)InstancesCountQuery.java	1.7 98/10/08
 *
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package hat.server;

import hat.model.*;
import hat.util.ArraySorter;
import hat.util.Comparer;

/**
 *
 * @version     1.7, 10/08/98
 * @author      Bill Foote
 */


class InstancesCountQuery extends QueryHandler {


    public InstancesCountQuery() {
    }

    public void run() {
	startHtml("Instance Counts for All Classes");

	JavaClass[] classes = snapshot.getClasses();
	ArraySorter.sort(classes, new Comparer() {
	    public int compare(Object lhso, Object rhso) {
		JavaClass lhs = (JavaClass) lhso;
		JavaClass rhs = (JavaClass) rhso;
		int diff = lhs.getInstancesCount(false) 
				- rhs.getInstancesCount(false);
		if (diff != 0) {
		    return -diff;	// Sort from biggest to smallest
		}
		String left = lhs.getName();
		String right = rhs.getName();
		if (left.startsWith("[") != right.startsWith("[")) {
		    // Arrays at the end
		    if (left.startsWith("[")) {
			return 1;
		    } else {
			return -1;
		    }
		}
		return left.compareTo(right);
	    }
	});

	String lastPackage = null;
	long totalSize = 0;
	long instances = 0;
	for (int i = 0; i < classes.length; i++) {
	    JavaClass clazz = classes[i];
	    int count = clazz.getInstancesCount(false);
	    print("" + count);
	    printAnchorStart();
	    out.print("instances/" + classes[i].getName());
	    out.print("\"> ");
	    if (count == 1) {
		print("instance");
	    } else {
		print("instances");
	    }
	    out.print("</a> ");
	    if (snapshot.getHasNewSet()) {
		JavaHeapObject[] objects = clazz.getInstances(false);
		int newInst = 0;
		for (int j = 0; j < objects.length; j++) {
		    if (objects[j].isNew()) {
			newInst++;
		    }
		}
		print("(");
		printAnchorStart();
		out.print("newInstances/" + classes[i].getName());
		out.print("\">");
		print("" + newInst + " new");
		out.print("</a>) ");
	    }
	    print("of ");
	    printClass(classes[i]);
	    out.println("<br>");
	    instances += count;
	    totalSize += classes[i].getTotalInstanceSize();
	}
	out.println("<h2>Total of " + instances + " instances occupying " + totalSize + " bytes.</h2>");

	out.println("<h2>Other Queries</h2>");
	out.println("<ul>");
	out.println("<li>");
	printAnchorStart();
	out.print("\">");
	print("Show All Classes");
	out.println("</a>");
	out.println("</ul>");

	endHtml();
    }

    
}
